//  DtL - Derive to LaTeX by Gary Bishop 29 June 1999

#include <stdio.h>
#include "exp.h"

static char rcsid[] = "$Id: exp.cpp,v 1.1 1999-07-01 11:03:08-04 gb Exp gb $";

extern Str mathrm(const Str& s);
extern Str mathit(const Str& s);

Str exp::paren(int lv)
{
  Str result;
  if(level < lv)
    result += "\\left(";
  result += latex();
  if(level < lv)
    result += "\\right)";
  return result;
}

Str exp_var::latex()
{
  // format the name for use in LaTeX. This should be easy but names with embeded greek letters
  // can be tricky
  return mathit(name);
}

Str exp_const::latex()
{
  return val;
}

Str exp_list::latex()
{
  Str result;
  if(ecnt == 1 && head->etype == evector) {
    result = head->latex();
  } else {
    result = "\\left(";
    result += head->latex();
    for(exp_list *p = tail; p != 0; p = p->tail) {
      result += "," + p->head->latex();
    }
    result += "\\right)";
  }
  return result;
}

Str exp_call::latex()
{
  // format the function name, again handle imbeded greek characters
  return mathrm(fun) + "{" + args->latex() + "}";
}

Str exp_unary::latex()
{
  return op + opr->paren(level);
}

Str exp_binary::latex()
{
  return left->paren(level) + op + right->paren(level);
}
  
Str exp_times::latex()
{
  return left->paren(level) + " " + right->paren(level);
}

Str exp_divide::latex()
{
  return "\\frac{" + left->paren(0) + "}{" + right->paren(0) + "}";
}

Str exp_vector::latex()
{
  Str result;

  // see if this is really a matrix
  int ncols = 0;
  if(elems->head->etype == evector) { // yes it is a matrix
    int nrows = elems->ecnt; // each of my elements is a row
    exp_list *rlst = elems;
    exp_vector *row = (exp_vector*)rlst->head;
    int ncols = row->elems->ecnt;
    result += "\\left[\\begin{array}{" + Str("c") * ncols + "}\n";
    for(int j=0; j<nrows; j++) {
      exp_list* lst = row->elems;
      result += lst->head->latex();
      while(lst->tail != 0) {
	lst = lst->tail;
	result += " & " + lst->head->latex();
      }
      rlst = rlst->tail;
      if(rlst != 0) {
	result += "\\\\\n";
	row = (exp_vector*)rlst->head;
      }
    }
    result += "\\end{array}\\right]";
  } else { // it is a vector
    int ncols = elems->ecnt;
    result += "\\left[\\begin{array}{" + Str("c") * ncols + "}\n";
    exp_list* lst = elems;
    result += lst->head->latex();
    while(lst->tail != 0) {
      lst = lst->tail;
      result += " & " + lst->head->latex();
    }
    result += "\\end{array}\\right]";
  }
  return result;
}

Str exp_limit::latex()
{
  return "\\lim_{" + args->tail->head->latex() + "\\rightarrow " 
    + args->tail->tail->head->latex() + "}" + args->head->latex();
}

Str exp_integral::latex()
{
  Str result;
  if(args->ecnt <= 3) { // indefinate integral
    result = "\\int " + args->head->latex() + " d" + args->tail->head->latex();
    if(args->ecnt == 3) {
      result += " + " + args->tail->tail->head->latex();
    }
  } else if(args->ecnt == 4) { // definate integral
    result = "\\int_{" + args->tail->tail->head->latex() + "}^{"
      + args->tail->tail->tail->head->latex() + "}" + args->head->latex() + "\\:d" 
      + args->tail->head->latex();
  }
  return result;
}

Str exp_sumprod::latex()
{
  Str result;
  if(args->ecnt == 1) {
    result = op + args->head->latex();
  } else if(args->ecnt == 2) {
    Str var = args->tail->head->latex();
    if(var.length() == 1) {
      result = op + "_" + var + " " + args->head->latex();
    } else {
      result = op + "_{" + var + "}" + args->head->latex();
    }
  } else if(args->ecnt == 4) {
    result = op + "_{" + args->tail->head->latex() + "=" + args->tail->tail->head->latex() + "}^{"
      + args->tail->tail->tail->head->latex() + "} " + args->head->latex();
  }
  return result;
}

Str exp_script::latex()
{
  Str vc = vec->paren(70);
  Str ss = script->latex();
  if(ss.length() == 1) {
    return vc + op + ss;
  } else {
    return vc + op + "{" + ss + "}";
  }
}

Str exp_bracket::latex()
{
  return lbracket + operand->latex() + rbracket;
}
