%{
//  DtL - Derive to LaTeX by Gary Bishop 29 June 1999

static char rcsid[] = "$Id: scanner.l,v 1.2 1999-07-02 16:15:24-04 gb Exp gb $";

/* a scanner for Derive internal text representation */
/* I'll assume the internal representation which appears to just be indicies into the DfW font.
   I'll have to translate when reading a file because they appear to be in ASCII 8 bit code
*/

#include "exp.h"
#include "parser.h"

extern int yydebug; // a global flag to control printing for debugging

extern char* strsave(const char*);
extern Str symbol(Str);
extern Str function(Str);

int lineno = 1;   // global to count lines for error messages

// I need my own input routine to handle the clipboard and the funky end-of-line continuations
// in .mth files.

int DtL_input(char*, int);
#define YY_INPUT(buf,result,max_size) { result=DtL_input(buf,max_size); }
%}
%option noyywrap


DIGIT	[0-9]
ID	[a-zA-Z_\251\337\252\253\255\256\257\262\263\264\277\265\376\270\271\272\336\274\275\276\247\367\233\235\236\241\242\243\244\320\245\327\250][a-zA-Z_0-9\251\337\252\253\255\256\257\262\263\264\277\265\376\270\271\272\336\274\275\276\247\367\233\235\236\241\242\243\244\320\245\327\250]*

%%
{DIGIT}+	{ if(yydebug)
                    fprintf(stderr, "NUM=%s\n", yytext);
                    yylval.s = strsave(yytext);
                    return NUM;
                }
{DIGIT}+"."{DIGIT}*	{ if(yydebug) fprintf(stderr, "NUM=%s\n", yytext);
                          yylval.s = strsave(yytext);
                          return NUM;
                        }
SQRT|\213	{ if(yydebug) fprintf(stderr, "SQRT\n");
		  return SQRT;
		}
FLOOR		{ if(yydebug) fprintf(stderr, "FLOOR\n"); return FLOOR; }
CEILING		{ if(yydebug) fprintf(stderr, "CEILING\n"); return CEILING; }
ABS		{ if(yydebug) fprintf(stderr, "ABS\n"); return ABS; }
AND|\217        { if(yydebug) fprintf(stderr, "AND\n"); return AND; }
OR|\220		{ if(yydebug) fprintf(stderr, "OR\n"); return OR; }
LEQ|"<="|\223	{ if(yydebug) fprintf(stderr, "LEQ\n"); return LEQ; }
GEQ|">="|\222	{ if(yydebug) fprintf(stderr, "LEQ\n"); return LEQ; }
NEQ|"/="|\224	{ if(yydebug) fprintf(stderr, "NEQ\n"); return NEQ; }
INT             { if(yydebug) fprintf(stderr, "INT\n"); return INT; }
DIF		{ if(yydebug) fprintf(stderr, "DIF\n"); return DIF; }
SUM|\244	{ if(yydebug) fprintf(stderr, "SUM\n"); return SUM; }
PRODUCT|\243	{ if(yydebug) fprintf(stderr, "PRODUCT\n"); return PROD; }
LIM		{ if(yydebug) fprintf(stderr, "LIM\n"); return LIM; }
SUB|\231        { if(yydebug) fprintf(stderr, "SUB\n"); return SUB; }
\226		{ if(yydebug) fprintf(stderr, "inf\n"); yylval.s = "inf"; return VAR; }
"#e"|\352	{ if(yydebug) fprintf(stderr, "#e\n"); yylval.s = "\352"; return VAR; }
"#i"|\356	{ if(yydebug) fprintf(stderr, "#i\n"); yylval.s = "\356"; return VAR; }
{ID}		{ if(yydebug)
                    fprintf(stderr, "VAR='%s' %o", yytext, yytext[0]&0xff);
                  yylval.s = strsave(yytext);
                  return VAR;
                }
":"epsilon      { if(yydebug) fprintf(stderr, "OP=:epsilon\n"); return ELEMENTOF; }
"+"|"-"|"/"|"^" { if(yydebug) fprintf(stderr, "OP=%s\n", yytext); return yytext[0]; }
"*"|\267|"."    { if(yydebug) fprintf(stderr, "OP=*\n"); return '*'; }
","|"["|"]"     { if(yydebug) fprintf(stderr, "OP=%s\n", yytext); return yytext[0]; }
">"|"<"         { if(yydebug) fprintf(stderr, "OP=%s\n", yytext); return yytext[0]; }
"'"             { if(yydebug) fprintf(stderr, "OP=APOSTROPHE\n"); return APOSTROPHE; }
"("|")"|"="|":"	{ if(yydebug) fprintf(stderr, "OP=%s\n", yytext); return yytext[0]; }
\n		{ lineno++; return EOL; }
[ \t]+		;/* eat up whitespace */
";".*\n		{ lineno++; yylval.s = strsave(yytext); return COMMENT; }
.		;/* eat up unrecognized junk */

%%
